@@ -6,10 +6,13 @@
 #ifndef KATE_INDENT_DETECTER_H
 #define KATE_INDENT_DETECTER_H
 
-#include <katedocument.h>
+namespace KTextEditor
+{
+class DocumentPrivate;
+};
 
 /**
- * File indentation detecter. Mostly ported from VSCode to here
+ * File indentation detector. Mostly ported from VSCode to here
  */
 class KateIndentDetecter
 {
