@@ -17,7 +17,7 @@ class KConfigGroup;
 
 namespace KateVi
 {
-class KTEXTEDITOR_EXPORT Macros
+class Macros
 {
 public:
     explicit Macros();
@@ -28,7 +28,7 @@ public:
 
     void store(const QChar &reg, const QList<KeyEvent> &macroKeyEventLog, const CompletionList &completions);
     void remove(const QChar &reg);
-    void clear();
+    KTEXTEDITOR_EXPORT void clear();
 
     QString get(const QChar &reg) const;
     CompletionList getCompletions(const QChar &reg) const;
